==> arclet-alconna-1.6.2.tar.gz: arclet-alconna-1.6.2/src/arclet/alconna/duplication.py <==
from __future__ import annotations

from inspect import isclass
from typing import cast

from .arparma import Arparma
from .stub import ArgsStub, BaseStub, OptionStub, SubcommandStub


class Duplication:
    """用以更方便的检查、调用解析结果的类。"""
    header: dict[str, str]

    def __init__(self, target: Arparma):
        from .base import Option, Subcommand
        self.header = target.header.copy()
        for key, value in self.__annotations__.items():
            if isclass(value) and issubclass(value, BaseStub):
                if value == ArgsStub:
                    setattr(self, key, ArgsStub(target.source.args).set_result(target.main_args))
                elif value == SubcommandStub:
                    for subcommand in filter(lambda x: isinstance(x, Subcommand), target.source.options):
                        if subcommand.dest == key and key in target.subcommands:
                            setattr(self, key, SubcommandStub(subcommand).set_result(target.subcommands[key]))
                elif value == OptionStub:
                    for option in filter(lambda x: isinstance(x, Option), target.source.options):
                        if option.dest == key and key in target.options:
                            setattr(self, key, OptionStub(option).set_result(target.options[key]))
            elif key != 'header' and key in target.all_matched_args:
                setattr(self, key, target.all_matched_args[key])

    def __repr__(self):
        return f'{self.__class__.__name__}({self.__annotations__})'

    def option(self, name: str) -> OptionStub | None:
        return cast(OptionStub, getattr(self, name, None))

    def subcommand(self, name: str) -> SubcommandStub | None:
        return cast(SubcommandStub, getattr(self, name, None))


def generate_duplication(arp: Arparma) -> Duplication:
    """依据给定的命令生成一个解析结果的检查类。"""
    from .base import Option, Subcommand
    options = filter(lambda x: isinstance(x, Option), arp.source.options)
    subcommands = filter(lambda x: isinstance(x, Subcommand), arp.source.options)
    return cast(Duplication, type(
        f"{arp.source.name.strip('/.-:')}Interface",
        (Duplication,), {
            "__annotations__": {
                "args": ArgsStub,
                **{opt.dest: OptionStub for opt in options},
                **{sub.dest: SubcommandStub for sub in subcommands},
            }
        }
    )(arp))

==> arclet-alconna-1.7.0rc2.tar.gz: arclet-alconna-1.7.0rc2/src/arclet/alconna/duplication.py <==
from __future__ import annotations

from inspect import isclass
from typing import cast
from tarina import Empty

from .arparma import Arparma
from .stub import ArgsStub, BaseStub, OptionStub, SubcommandStub


class Duplication:
    """用以更方便的检查、调用解析结果的类。"""
    header: dict[str, str]

    def __init__(self, target: Arparma):
        from .base import Option, Subcommand
        self.header = target.header.copy()
        for key, value in self.__annotations__.items():
            if isclass(value) and issubclass(value, BaseStub):
                if value == ArgsStub:
                    setattr(self, key, ArgsStub(target.source.args).set_result(target.main_args))
                elif value == SubcommandStub:
                    for subcommand in filter(lambda x: isinstance(x, Subcommand), target.source.options):
                        if subcommand.dest == key:
                            setattr(self, key, SubcommandStub(subcommand).set_result(target.subcommands.get(key, None)))
                elif value == OptionStub:
                    for option in filter(lambda x: isinstance(x, Option), target.source.options):
                        if option.dest == key:
                            setattr(self, key, OptionStub(option).set_result(target.options.get(key, None)))
            elif key != 'header':
                setattr(self, key, target.all_matched_args.get(key, Empty))

    def __repr__(self):
        return f'{self.__class__.__name__}({self.__annotations__})'

    def option(self, name: str) -> OptionStub | None:
        return cast(OptionStub, getattr(self, name, None))

    def subcommand(self, name: str) -> SubcommandStub | None:
        return cast(SubcommandStub, getattr(self, name, None))


def generate_duplication(arp: Arparma) -> Duplication:
    """依据给定的命令生成一个解析结果的检查类。"""
    from .base import Option, Subcommand
    options = filter(lambda x: isinstance(x, Option), arp.source.options)
    subcommands = filter(lambda x: isinstance(x, Subcommand), arp.source.options)
    return cast(Duplication, type(
        f"{arp.source.name.strip('/.-:')}Interface",
        (Duplication,), {
            "__annotations__": {
                "args": ArgsStub,
                **{opt.dest: OptionStub for opt in options},
                **{sub.dest: SubcommandStub for sub in subcommands},
            }
        }
    )(arp))
